
MyEventHandler = EVENTHANDLER:New()
EscortMonitor = EVENTHANDLER:New()


spawnedGroups = {}
local escortGroups = {}
local menuEscortRequest = {}
local escortRequestMenus = {}
local escortMenus = {}


local groupSet = SET_GROUP:New():FilterPrefixes("A-10C"):FilterAlive():FilterStart()

function AddEscortRequestMenu(group)
    if not group then
        env.info("AddEscortRequestMenu: Group is nil, cannot create menu.")
        return
    end
    local groupName = group:GetName()
    escortRequestMenus[groupName] = MENU_GROUP_COMMAND:New(group, "Запрос на сопровождение", nil, EscortClientGroup, group)
end

function EnableEscortRequestMenu(group)
    if not group then
        env.info("EnableEscortRequestMenu: Group is nil, cannot enable menu.")
        return
    end
    local groupName = group:GetName()
    if menuEscortRequest[groupName] then
        menuEscortRequest[groupName]:Remove()
    end
end

function RequestEscort(group)
    EscortClientGroup(group)
    local groupName = group:GetName()
    if menuEscortRequest[groupName] then
        menuEscortRequest[groupName]:Remove()
        menuEscortRequest[groupName] = nil
    end
end



function RemoveRequestEscortMenu(group)
    local groupName = group:GetName()
    if escortRequestMenus[groupName] then
        escortRequestMenus[groupName]:Remove()
        escortRequestMenus[groupName] = nil
    end
end

function FindEscortTemplateWithAlias(alias)
    local escortSpawn = SPAWN:NewWithAlias("EscortA10", alias)
    return escortSpawn
end

function EscortClientGroup(clientGroup)
    local groupName = clientGroup:GetName()
    local alias = groupName .. "_Escort"
    local escortSpawn = FindEscortTemplateWithAlias(alias)
    
    local clientPosition = clientGroup:GetPointVec3()
    local clientHeading = clientGroup:GetHeading()
    
	local distanceBehindMeters = 5000
    local offsetX = math.cos(math.rad(clientHeading)) * distanceBehindMeters
    local offsetZ = math.sin(math.rad(clientHeading)) * distanceBehindMeters
        
    local escortSpawnPosition = {
        x = clientPosition.x - offsetX,
        y = clientPosition.y + 3700,
        z = clientPosition.z - offsetZ
    }
		
    local escortCoord = COORDINATE:NewFromVec3(escortSpawnPosition, clientHeading)
		
		escortSpawn:OnSpawnGroup(function(spawnedEscortGroup)

        local escortGroup = FLIGHTGROUP:New(spawnedEscortGroup)
		escortGroup:GetGroup():CommandSetUnlimitedFuel(true)
		
        --escortGroup:Teleport(escortCoord,2,true)
        escortGroups[groupName] = escortGroup
		
        local escortAuftrag = AUFTRAG:NewESCORT(clientGroup, {x=-100, y=3048, z=300}, 40, {"Air"})
        escortAuftrag:SetEPLRS(true)
		escortGroup:AddWaypoint(clientPosition, 480, nil, 25000, true)
        MESSAGE:New("ЭСКОРТ В ПУТИ.", 20):ToGroup(clientGroup)
		escortAuftrag:SetMissionWaypointCoord(clientPosition)
        escortGroup:AddMission(escortAuftrag)
		escortGroup:MissionStart(escortAuftrag)
        RemoveRequestEscortMenu(clientGroup)
        AddEscortMenu(clientGroup)
    end)
    
    escortSpawn:SpawnFromCoordinate(escortCoord)
	
end

function AddEscortMenu(group)
    if not group then
        env.info("AddEscortMenu: Group is nil, cannot create menu.")
        return
    end

    local menuEscort = MENU_GROUP:New(group, "Эскорт")
    MENU_GROUP_COMMAND:New(group, "Стартовая орбита", menuEscort, EscortOrbit, group)
    MENU_GROUP_COMMAND:New(group, "Направление, С севера на юг", menuEscort, RaceTrackNorthSouth, group)
	MENU_GROUP_COMMAND:New(group, "Направление, С востока на запад", menuEscort, RaceTrackWestEast, group)
    MENU_GROUP_COMMAND:New(group, "Присоединяйтесь к", menuEscort, EscortRejoin, group)
	MENU_GROUP_COMMAND:New(group, "Эскорт RTB", menuEscort, EscortAbort, group)
end

function RemoveEscortMenu(group)
    local groupName = group:GetName()
    if escortMenus[groupName] then
        escortMenus[groupName]:Remove()
        escortMenus[groupName] = nil
        env.info("Меню сопровождения для " .. groupName .. " был удалён.")
    end
end

function EscortOrbit(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        local clientCoord = group:GetPointVec2()
        local escortHeading = group:GetHeading()

		escortGroup:AddWaypoint(clientCoord, 480, nil, 25000, true)
		
        local orbitAuftrag = AUFTRAG:NewORBIT(clientCoord, 25000, 312)
        orbitAuftrag:SetMissionWaypointCoord(clientCoord)
        orbitAuftrag:SetMissionSpeed(480)
        orbitAuftrag:SetMissionAltitude(25000)
        
        escortGroup:AddMission(orbitAuftrag)
        
        local currentMission = escortGroup:GetMissionCurrent()
        if currentMission then
            escortGroup:MissionCancel(currentMission)
        end
        
        escortGroup:MissionStart(orbitAuftrag)
        
        MESSAGE:New("Эскорт выводит на орбиту.", 20):ToGroup(group)
    else
        MESSAGE:New("Активный эскорт не найден.", 10):ToGroup(group)
    end
end


function RaceTrackNorthSouth(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        local clientCoord = group:GetPointVec3()
		
        local RaceTracknorthtosouthAuftrag = AUFTRAG:NewPATROL_RACETRACK(clientCoord, 25000, 320, 0, 10)
        
		RaceTracknorthtosouthAuftrag:SetMissionSpeed(450)
		RaceTracknorthtosouthAuftrag:SetMissionAltitude(25000)
        escortGroup:AddMission(RaceTracknorthtosouthAuftrag)
        		
		local currentMission = escortGroup:GetMissionCurrent()
        
        
        if currentMission then
            escortGroup:MissionCancel(currentMission)
        end
		escortGroup:MissionStart(RaceTracknorthtosouthAuftrag)
        
        MESSAGE:New("Ескорд создает трассу длиной 10 НМ с севера на юг..", 20):ToGroup(group)
    else
        MESSAGE:New("Активный эскорт не найден.", 10):ToGroup(group)
    end
end
function RaceTrackWestEast(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        local clientCoord = group:GetPointVec3()
		
        local RaceTrackwesttosouthAuftrag = AUFTRAG:NewPATROL_RACETRACK(clientCoord, 25000, 320, 090, 10)
        RaceTrackwesttosouthAuftrag:SetMissionSpeed(600)
		RaceTrackwesttosouthAuftrag:SetMissionAltitude(25000)
        escortGroup:AddMission(RaceTrackwesttosouthAuftrag)
        		
		local currentMission = escortGroup:GetMissionCurrent()
        
        
        if currentMission then
            escortGroup:MissionCancel(currentMission)
        end
        escortGroup:MissionStart(RaceTrackwesttosouthAuftrag)
        
        MESSAGE:New("Ескорд создает трассу длиной 10 НМ с востока на запад.", 20):ToGroup(group)
    else
        MESSAGE:New("Активный эскорт не найден.", 20):ToGroup(group)
    end
end

function EscortRejoin(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        escortGroup:CancelAllMissions()
		local clientCoord = group:GetPointVec3()

		
        local escortAuftrag = AUFTRAG:NewESCORT(group, {x=-100, y=3048, z=300}, 40, {"Air"})
		
        escortAuftrag:SetMissionSpeed(450)
		escortAuftrag:SetMissionAltitude(25000)
		escortGroup:AddMission(escortAuftrag)
		escortGroup:MissionStart(escortAuftrag)
		
        
        
        MESSAGE:New("Эскорт присоединяется к вашему строю.", 20):ToGroup(group)
    else
        MESSAGE:New("Активный эскорт не найден.", 10):ToGroup(group)
    end
end


function EscortAbort(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
                
        escortGroup:CancelAllMissions()
        
        
        MESSAGE:New("Сопровождение - RTB", 20):ToGroup(group)
    else
        MESSAGE:New("Активный эскорт не найден.", 10):ToGroup(group)
    end
end

function MyEventHandler:OnEventTakeoff(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit -- Wrapper.Unit#UNIT
        local playerGroup = playerUnit:GetGroup()
        local PGName = playerGroup:GetName()
         
        groupSet:ForEachGroup(function(group)
            if group:GetName() == PGName then
                local groupName = group:GetName()

                if not spawnedGroups[groupName] then
                    spawnedGroups[groupName] = {
                        playerName = EventData.IniPlayerName,
                        escortGroups = {},
                        menuEscortRequest = nil
                    }

                    env.info("Пилот " .. EventData.IniPlayerName .. " взлетел с группой: " .. groupName)
                    
                    MESSAGE:New("Сопровождение доступно, " .. EventData.IniPlayerName .. ".", 10, ""):ToUnit(playerUnit)

                    AddEscortRequestMenu(group)
                    menuEscortRequest[groupName] = spawnedGroups[groupName].menuEscortRequest
                end
            end
        end)
    end
end

function MyEventHandler:OnEventPlayerLeaveUnit(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local playerGroup = playerUnit:GetGroup()

        if playerGroup then
            local groupName = playerGroup:GetName()
            local escortGroup = escortGroups[groupName]

            if escortGroup then
                escortGroup:Destroy()
                escortGroups[groupName] = nil
                env.info("Группа сопровождения для " .. groupName .. " был уничтожен, потому что пилот покинул отряд.")
            end
        end
    end
end

function MyEventHandler:OnEventLand(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local playerGroup = playerUnit:GetGroup()

        if playerGroup then
            local groupName = playerGroup:GetName()
            local escortGroup = escortGroups[groupName]

            if escortGroup then
                escortGroup:Destroy()
                escortGroups[groupName] = nil
                env.info("Группа сопровождения для " .. groupName .. " был уничтожен, потому что пилот приземлился.")
            end
        end
    end
end
function MyEventHandler:OnEventPilotDead(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local playerGroup = playerUnit:GetGroup()

        if playerGroup then
            local groupName = playerGroup:GetName()
            local escortGroup = escortGroups[groupName]

            if escortGroup then
                escortGroup:Destroy()
                escortGroups[groupName] = nil
                env.info("Группа сопровождения для " .. groupName .. " был уничтожен, потому что пилот приземлился.")
            end
        end
    end
end



function EscortMonitor:OnEventDead(EventData)
    if EventData.IniGroup then
        local escortGroupName = EventData.IniGroup:GetName()
        
        if escortGroups[escortGroupName] then
            env.info("Группа сопровождения " .. escortGroupName .. " была уничтожена.")

			escortGroups[escortGroupName] = nil

            
            local clientGroup = EventData.IniGroup
            MESSAGE:New("Ваша группа сопровождения была уничтожена\nПриземлившись, вы получите новую группу сопровождения, готовую к взлету..", 10):ToGroup(clientGroup)
			
			RemoveEscortMenu(clientGroup)
        end
    end
end





MyEventHandler:HandleEvent(EVENTS.PlayerLeaveUnit)
MyEventHandler:HandleEvent(EVENTS.Land)
MyEventHandler:HandleEvent(EVENTS.Takeoff)
MyEventHandler:HandleEvent(EVENTS.PilotDead)
EscortMonitor:HandleEvent(EVENTS.Dead)
